@@ -28,6 +28,8 @@
 #include <QDBusConnection>
 #include <QDBusInterface>
 
+#include <qpa/qwindowsysteminterface.h>
+
 #include <KSandbox>
 #include <kcolorscheme.h>
 #include <kconfiggroup.h>
@@ -137,6 +139,8 @@ KHintsSettings::KHintsSettings(const KSharedConfig::Ptr &kdeglobals)
     QMetaObject::invokeMethod(this, "setupIconLoader", Qt::QueuedConnection);
 
     loadPalettes();
+    m_colorScheme = determineColorScheme();
+
     updateCursorTheme();
 }
 
@@ -250,6 +254,12 @@ void KHintsSettings::slotNotifyChange(int type, int arg)
         } else if (qobject_cast<QGuiApplication *>(QCoreApplication::instance())) {
             QGuiApplication::setPalette(*m_palettes[QPlatformTheme::SystemPalette]);
         }
+
+        Qt::ColorScheme newColorScheme = determineColorScheme();
+        if (m_colorScheme != newColorScheme) {
+            m_colorScheme = newColorScheme;
+            QWindowSystemInterface::handleThemeChange();
+        }
         break;
     }
     case SettingsChanged: {
@@ -441,6 +451,24 @@ void KHintsSettings::loadPalettes()
     }
 }
 
+Qt::ColorScheme KHintsSettings::determineColorScheme() const
+{
+    Qt::ColorScheme colorScheme = Qt::ColorScheme::Unknown;
+
+    if (auto *systemPalette = m_palettes[QPlatformTheme::SystemPalette]) {
+        // Matches xdg-desktop-portal-kde readFdoColorScheme()
+        const int windowBackgroundGray = qGray(systemPalette->window().color().rgb());
+
+        if (windowBackgroundGray < 192) {
+            colorScheme = Qt::ColorScheme::Dark;
+        } else {
+            colorScheme = Qt::ColorScheme::Light;
+        }
+    }
+
+    return colorScheme;
+}
+
 void KHintsSettings::updateCursorTheme()
 {
     KSharedConfig::Ptr inputConfig = KSharedConfig::openConfig(QStringLiteral("kcminputrc"));
